from django.forms.utils import ErrorList
from django.utils.text import slugify
from wagtail import blocks
from wagtail.blocks import (
    CharBlock,
    DateTimeBlock,
    IntegerBlock,
    ListBlock,
    PageChooserBlock,
    RichTextBlock,
    StreamBlock,
    StructBlock,
    TextBlock,
    URLBlock,
)
from wagtail.blocks.struct_block import StructBlockValidationError
from wagtail.images.blocks import ImageChooserBlock

from shared.blocks import CandidateBlock as SharedCandidateBlockMixin
from shared.blocks import CandidateListBlock as SharedCandidateListBlockMixin
from shared.blocks import (
    CandidateSecondaryListBlock as SharedCandidateSecondaryListBlockMixin,
)
from shared.blocks import (
    CardLinkBlockMixin,
    CardLinkWithHeadlineBlockMixin,
    PersonContactBlockMixin,
    ProgramGroupBlock,
    ProgramGroupBlockMixin,
)
from shared.blocks import ProgramGroupBlockPopout as SharedProgramGroupBlockPopout
from shared.blocks import ProgramItemBlock
from shared.blocks import SecondaryCandidateBlock as SharedSecondaryCandidateBlockMixin

# --- BEGIN New blocks ---


class FullscreenHeaderBlock(StructBlock):
    desktop_image = ImageChooserBlock(
        label="Obrázek na pozadí (desktop)",
        help_text="Pokud není vybráno video, ukáže se na desktopu.",
        required=False,
    )

    mobile_image = ImageChooserBlock(
        label="Obrázek na pozadí (mobil)",
        help_text="Pokud není vybráno video, ukáže se na mobilu.",
        required=False,
    )

    desktop_video_url = URLBlock(
        label="Video (desktop)",
        help_text="Pokud je vybráno, ukáže se na desktopech s povoleným autoplayem místo obrázku.",
        required=False,
    )

    mobile_video_url = URLBlock(
        label="Video (mobil)",
        help_text="Pokud je vybráno, ukáže se na mobilech s povoleným autoplayem místo obrázku.",
        required=False,
    )

    desktop_line_1 = TextBlock(label="Desktop první řádek")
    desktop_line_2 = TextBlock(label="Desktop druhý řádek", required=False)

    mobile_line_1 = TextBlock(label="První mobilní řádek")
    mobile_line_2 = TextBlock(label="Druhý mobilní řádek", required=False)
    mobile_line_3 = TextBlock(label="Třetí mobilní řádek", required=False)

    button_url = URLBlock(
        label="Odkaz tlačítka",
        help_text="Bez odkazu tlačítko nebude viditelné.",
        required=False,
    )
    button_text = CharBlock(label="Text tlačítka", required=False)

    class Meta:
        template = "styleguide2/includes/molecules/menus/district/carousel.html"
        icon = "form"
        label = "Header"


class ButtonBlock(blocks.StructBlock):
    title = blocks.CharBlock(label="Titulek", max_length=128, required=True)

    page = blocks.PageChooserBlock(label="Stránka", required=False)

    link = blocks.URLBlock(label="Odkaz", required=False)

    class Meta:
        label = "Tlačítko"


class ElectionsCountdownBlock(StructBlock):
    title = CharBlock(label="Titulek", required=True)

    text_before_countdown = CharBlock(
        label="Text před odpočtem", default="Aktuálně zbývá", required=True
    )

    countdown_timestamp = DateTimeBlock(label="Datum & čas voleb", required=True)

    buttons = ListBlock(ButtonBlock(), label="Tlačítka", required=False)

    class Meta:
        label = "Odpočet do voleb"
        icon = "time"
        template = "styleguide2/includes/organisms/main_section/district/elections.html"


class CalendarBlock(StructBlock):
    title = CharBlock(label="Titulek", required=False)

    class Meta:
        label = "Kalendář"
        icon = "calendar-alt"
        template = "styleguide2/includes/organisms/main_section/district/calendar.html"


class NewsletterBlock(StructBlock):
    class Meta:
        label = "Newsletter banner"
        icon = "mail"
        template = "styleguide2/includes/organisms/main_section/newsletter_section.html"


class PeopleGroupBlock(StructBlock):
    title = CharBlock(label="Titulek")
    slug = CharBlock(
        label="Slug skupiny",
        required=False,
        help_text="Není třeba vyplňovat, bude automaticky vyplněno",
    )
    person_list = ListBlock(
        PageChooserBlock(page_type="district.DistrictPersonPage", label="Detail osoby"),
        label="Skupina osob",
    )

    class Meta:
        icon = "group"
        label = "Slug skupiny"

    def get_prep_value(self, value):
        value = super().get_prep_value(value)
        value["slug"] = slugify(value["title"])
        return value


class CardLinkBlock(CardLinkBlockMixin):
    page = PageChooserBlock(
        label="Stránka",
        page_type=[
            "district.DistrictArticlesPage",
            "district.DistrictCenterPage",
            "district.DistrictContactPage",
            "district.DistrictCrossroadPage",
            "district.DistrictCustomPage",
            "district.DistrictPeoplePage",
            "district.DistrictProgramPage",
            "district.DistrictInteractiveProgramPage",
            "district.DistrictGeoFeatureCollectionPage",
            "district.DistrictCalendarPage",
            "district.DistrictPdfPage",
            "district.DistrictNewProgramPage",
        ],
        required=False,
    )

    class Meta:
        template = "styleguide2/includes/molecules/boxes/card_box_block.html"
        icon = "link"
        label = "Karta s odkazem"


class CardLinkWithHeadlineBlock(CardLinkWithHeadlineBlockMixin):
    card_items = ListBlock(
        CardLinkBlock(
            template="styleguide2/includes/molecules/boxes/card_box_block.html"
        ),
        label="Karty s odkazy",
    )

    class Meta:
        template = (
            "styleguide2/includes/molecules/boxes/card_box_with_headline_block.html"
        )
        icon = "link"
        label = "Karty odkazů s nadpisem"


class TeamBlock(StructBlock):
    title = CharBlock(label="Název sekce týmů")
    slug = CharBlock(
        label="Slug sekce",
        required=False,
        help_text="Není třeba vyplňovat, bude automaticky vyplněno",
    )
    team_list = ListBlock(
        CardLinkWithHeadlineBlock(label="Karta týmu"),
        label="Týmy",
    )

    class Meta:
        icon = "group"
        label = "Týmy"

    def get_prep_value(self, value):
        value = super().get_prep_value(value)
        value["slug"] = slugify(value["title"])
        return value


class AddressBlock(StructBlock):
    title = CharBlock(label="Titulek", required=True)
    map_link = URLBlock(label="Odkaz na detail mapy", required=False)
    address = TextBlock(label="Adresa", required=True)
    address_info = TextBlock(label="Info k adrese", required=False)

    class Meta:
        template = "styleguide2/includes/atoms/text/address.html"
        icon = "home"
        label = "Adresa"


class PersonContactBlock(PersonContactBlockMixin):
    person = PageChooserBlock(
        label="Osoba",
        page_type=["district.DistrictPersonPage"],
    )


class CenterContactBlock(StructBlock):
    title = CharBlock(label="Titulek", required=True)
    contact_list = ListBlock(PersonContactBlock())

    class Meta:
        template = "styleguide2/includes/molecules/contact/center_contacts.html"
        icon = "mail"
        label = "Kontakt"


# --- END New blocks ---


class SimplePersonBlock(StructBlock):
    title = blocks.CharBlock(label="Jméno a příjmení", max_length=128, required=True)
    job = blocks.CharBlock(
        label="Povolání", max_length=128, required=False, help_text="Např. 'Informatik'"
    )
    profile_photo = ImageChooserBlock(label="Profilová fotka", required=False)
    email = blocks.EmailBlock(label="Email", required=False)
    city = blocks.CharBlock(label="Město/obec", max_length=64, required=False)
    age = blocks.IntegerBlock(label="Věk", required=False)
    is_pirate = blocks.BooleanBlock(
        label="Je členem Pirátské strany?", default=True, required=False
    )
    other_party = blocks.CharBlock(label="Strana", max_length=64, required=False)
    other_party_logo = ImageChooserBlock(
        label="Logo strany", required=False, help_text="Vyplňte pokud osoba není Pirát"
    )

    # socials
    facebook_url = blocks.URLBlock(label="Odkaz na Facebook", required=False)
    instagram_url = blocks.URLBlock(label="Odkaz na Instagram", required=False)
    twitter_url = blocks.URLBlock(label="Odkaz na Twitter", required=False)
    youtube_url = blocks.URLBlock(label="Odkaz na Youtube kanál", required=False)
    flickr_url = blocks.URLBlock(label="Odkaz na Flickr", required=False)

    class Meta:
        icon = "user"
        label = "Osoba blok"


class CardLinkBlock(CardLinkBlockMixin):
    page = PageChooserBlock(
        label="Stránka",
        page_type=[
            "district.DistrictArticlePage",
            "district.DistrictArticlesPage",
            "district.DistrictCenterPage",
            "district.DistrictContactPage",
            "district.DistrictCrossroadPage",
            "district.DistrictCustomPage",
            "district.DistrictElectionCampaignPage",
            "district.DistrictElectionProgramPage",
            "district.DistrictPeoplePage",
            "district.DistrictPersonPage",
            "district.DistrictPostElectionStrategyPage",
            "district.DistrictProgramPage",
        ],
        required=False,
    )

    class Meta:
        template = "district/blocks/card_link_block.html"
        icon = "link"
        label = "Karta s odkazem"


class HomepageHeaderBlock(StructBlock):
    title = CharBlock(label="Titulek", required=False, help_text="Nap")
    image = ImageChooserBlock()
    button_text = CharBlock(label="Text tlačítka", required=False)
    button_link = URLBlock(label="Odkaz tlačítka", required=False)

    class Meta:
        template = "district/blocks/homepage_header_block.html"
        icon = "placeholder"
        label = "Nadpis s obrázkem a tlačítky"


class HomepageSimpleHeaderBlock(StructBlock):
    title = CharBlock(label="Titulek", required=False)
    subtitle = CharBlock(label="Podtitulek", required=False)
    image = ImageChooserBlock()

    class Meta:
        template = "district/blocks/homepage_simple_header_block.html"
        icon = "image"
        label = "Nadpis s obrázkem v pozadí"


class PersonBadgeBlock(StructBlock):
    person = PageChooserBlock(
        label="Osoba", page_type=("district.DistrictPersonPage",), required=True
    )
    caption = CharBlock(
        label="Popisek",
        required=False,
        help_text="Vlastní popisek na vizitce. Pokud není uvedeno, použije se výchozí profese osoby.",
    )

    class Meta:
        template = "styleguide2/includes/molecules/contact/person_badge.html"
        icon = "user"
        label = "Vizitka"


class PeopleGroupListBlock(StructBlock):
    group_title = CharBlock(label="Titulek", required=True)
    person_list = ListBlock(
        PersonContactBlock(),
        label="List osob",
    )

    class Meta:
        template = "district/blocks/people_group_block.html"
        icon = "list-ul"
        label = "Skupina členů"


class PersonUrlBlock(StructBlock):
    title = CharBlock(label="Název", required=True)
    url = URLBlock(label="URL", required=True)
    custom_icon = CharBlock(
        label="Vlastní ikonka ze styleguide",
        required=False,
        help_text="Pro vlastní ikonku zadejde název ikonky z https://styleguide.pirati.cz/latest/?p=viewall-atoms-icons (bez tečky), např. 'ico--beer'",
    )


class InteractiveProgramItemBlock(StructBlock):
    content = RichTextBlock(label="Obsah", required=True)


class InteractiveProgramBlock(StructBlock):
    name = CharBlock(label="Název části programu", required=True)

    heading = CharBlock(label="Nadpis", required=True)
    description = RichTextBlock(
        label="Krátký popis, BEZ BODŮ", required=False, blank=True
    )

    icon = blocks.ChoiceBlock(
        label="Ikona",
        required=True,
        choices=[
            ("verejne-sluzby", "Veřejné služby"),
            ("zdravotni-pece", "Zdravotní péče"),
            ("sport", "Sport"),
            ("sport-plavec", "Sport - plavec"),
            ("socialni-politika", "Sociální politika"),
            ("aktivni-verejnost", "Aktivní veřejnost"),
            ("priroda", "Příroda"),
            ("vzdelavani", "Vzdělávání"),
            ("kultura", "Kultura"),
            ("otevrena-radnice", "Otevřená radnice"),
            ("cestovni-ruch", "Cestovní ruch"),
            ("doprava", "Doprava"),
            ("bydleni", "Bydlení"),
        ],
    )

    program_items = ListBlock(
        InteractiveProgramItemBlock(), label="Seznam bodů", required=True
    )

    class Meta:
        template = "district/blocks/interactive_program_block.html"
        icon = "list-ul"
        label = "Část programu"


class StaticProgramBlock(StructBlock):
    headline = CharBlock(label="Titulek bloku", required=True)
    perex = TextBlock(label="Krátký text pod nadpisem", required=True)
    person = PersonContactBlock()
    completion_percentage = IntegerBlock(label="Procento dokončení", required=True)
    program_items = ListBlock(ProgramItemBlock(), label="Seznam bodů")

    class Meta:
        template = "district/blocks/program_block.html"
        icon = "list-ul"
        label = "Blok programu"


class RedmineProgramBlock(StructBlock):
    headline = CharBlock(label="Titulek bloku", required=True)
    perex = TextBlock(label="Krátký text pod nadpisem", required=True)
    person = PersonContactBlock()
    redmine_issue = IntegerBlock(label="Číslo Redmine issue", required=True)
    completion_percentage = IntegerBlock(
        label="Procento dokončení - bude doplněno automaticky",
        required=False,
        help_text="Hodnota se automaticky načte s Redmine",
    )
    program_items = ListBlock(
        ProgramItemBlock(),
        label="Seznam bodů - bude doplněno automaticky",
        help_text="Hodnota se automaticky načte s Redmine",
        required=False,
    )

    class Meta:
        template = "district/blocks/program_block.html"
        icon = "site"
        label = "Blok programu stahovaný z Redmine"


class HeroBannerBlock(StructBlock):
    sm_banner = ImageChooserBlock(
        label="Obrázek pro zobrazení menší než 640px", required=True
    )
    md_banner = ImageChooserBlock(
        label="Obrázek pro zobrazení větší než 768px", required=True
    )
    lg_banner = ImageChooserBlock(
        label="Obrázek pro zobrazení větší než 1024px", required=True
    )
    xl_banner = ImageChooserBlock(
        label="Obrázek pro zobrazení větší než 1280px", required=True
    )
    xxl_banner = ImageChooserBlock(
        label="Obrázek pro zobrazení větší než 1536px", required=True
    )

    title = CharBlock(label="Titulek bloku", required=False)
    subtitle = CharBlock(label="Podtitulek", required=False)
    button_text = CharBlock(label="Text tlačítka", required=False)
    button_link = URLBlock(label="Odkaz tlačítka", required=False)

    def clean(self, value):
        errors = {}

        if value["button_link"] and not value["button_text"]:
            errors["button_text"] = ErrorList(
                ["Zadejte prosím text tlačítka, nebo smažte odkaz v tlačítku"]
            )

        if not value["button_link"] and value["button_text"]:
            errors["button_link"] = ErrorList(
                ["Zadejte prosím odkaz tlačítka, nebo smažte text v tlačítku"]
            )

        if errors:
            raise StructBlockValidationError(errors)

        return super().clean(value)

    class Meta:
        template = "district/blocks/hero_banner_block.html"
        icon = "image"
        label = "Hero banner"


class ArticlesBlock(StructBlock):
    articles = ListBlock(
        PageChooserBlock(
            "district.DistrictArticlePage", required=True, label="Aktualita"
        ),
        label="Seznam aktualit",
        required=True,
    )

    class Meta:
        template = "styleguide2/includes/organisms/articles/article_list_block.html"
        icon = "list-ul"
        label = "Blok aktualit"


class ArticleLinksBlock(StructBlock):
    articles = ListBlock(
        PageChooserBlock(
            "district.DistrictArticlePage", required=True, label="Aktualita"
        ),
        label="Seznam aktualit",
        required=True,
    )

    class Meta:
        template = "styleguide2/includes/organisms/articles/article_links_block.html"
        icon = "list-ul"
        label = "Seznam nadpisů článků"


class ProgramGroupBlockCrossroad(ProgramGroupBlockMixin):
    point_list = ListBlock(CardLinkBlock(), label="Karty programu")

    class Meta:
        icon = "date"
        label = "Rozcestníkový program"


class ProgramGroupBlockPopout(SharedProgramGroupBlockPopout, ProgramGroupBlockMixin):
    pass


class CandidateBlock(SharedCandidateBlockMixin):
    page = PageChooserBlock(label="Stránka", page_type=["district.DistrictPersonPage"])


class SecondaryCandidateBlock(SharedSecondaryCandidateBlockMixin):
    page = PageChooserBlock(label="Stránka", page_type=["district.DistrictPersonPage"])


class CandidateListBlock(SharedCandidateListBlockMixin):
    candidates = ListBlock(
        CandidateBlock(),
        label=" ",  # Hacky way to show no label
    )


class CandidateSecondaryListBlock(SharedCandidateSecondaryListBlockMixin):
    candidates = ListBlock(
        SecondaryCandidateBlock(),
        label="Zbylí kandidáti na listině",
    )


class ProgramGroupWithCandidatesBlock(StructBlock):
    title = CharBlock(
        label="Název programu",
        help_text="Např. 'Krajské volby 2024', 'Evropské volby 2024', ...",
    )

    preamble_content = RichTextBlock(
        label="Preambule",
        help_text="Text, který se zobrazí před přepínačem mezi kandidáty a programem.",
        required=False,
    )

    primary_candidates = CandidateListBlock(
        label="Osoby na čele kandidátky",
        help_text="Zobrazí se ve velkých blocích na začátku stránky.",
    )

    secondary_candidates = CandidateSecondaryListBlock(
        label="Ostatní osoby na kandidátce",
        help_text="Zobrazí se v kompaktním seznamu pod čelem kandidátky.",
    )

    program = StreamBlock(
        [
            ("program_group", ProgramGroupBlock()),
            ("program_group_crossroad", ProgramGroupBlockCrossroad()),
            ("program_group_popout", ProgramGroupBlockPopout()),
        ]
    )

    class Meta:
        icon = "list-ul"
        label = "Program s kandidáty"
        template = "styleguide2/includes/molecules/program/program_block.html"

    def get_prep_value(self, value):
        value = super().get_prep_value(value)
        value["slug"] = slugify(value["title"])
        return value
